@@ -1,9 +1,9 @@
 # Copyright (C) 2008 One Laptop Per Child
 # Copyright (C) 2014 Sugar Labs, Frederick Grose
 #
-# This program is free software; you can redistribute it and/or modify
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -12,13 +12,13 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
 import logging
 
-import dbus
+import gi
+gi.require_version('NMClient', '1.0')
 from gettext import gettext as _
 from gi.repository import Gio
 from gi.repository import NMClient
@@ -82,7 +82,6 @@ def set_jabber(server):
 
 def get_radio():
     try:
-        nm_client = NMClient.Client()
         return nm_client.wireless_get_enabled()
     except:
         raise ReadError(_('State is unknown.'))
@@ -98,10 +97,9 @@ def set_radio(state):
     """
     try:
         state = state or state == 'on' or state == 1
-        nm_client = NMClient.Client()
         nm_client.wireless_set_enabled(state)
     except:
-        raise ValueError(_('Error in specified radio argument use on/off.'))
+        raise ValueError(_('Error in specified radio argument. Use on/off.'))
 
 
 def clear_registration():
@@ -117,63 +115,12 @@ def clear_registration():
     return 1
 
 
-wifi_whitelist = ('Sugar Ad-hoc Network 1',
-                  'Sugar Ad-hoc Network 6',
-                  'Sugar Ad-hoc Network 11')
-mesh_whitelist = ('OLPC Mesh Network 1',
-                  'OLPC Mesh Network 6',
-                  'OLPC Mesh Network 11',
-                  'OLPC XS Mesh Network 1',
-                  'OLPC XS Mesh Network 6',
-                  'OLPC XS Mesh Network 11')
-
-
-def is_wireless(connection):
-    """Check for wireless connection not whitelisted by Sugar.
-    """
-    wifi_settings = connection.get_settings(
-        network.NM_CONNECTION_TYPE_802_11_WIRELESS)
-    if wifi_settings:
-        return not (wifi_settings['mode'] == 'adhoc' and
-                    connection.get_id() in wifi_whitelist)
-
-    mesh_settings = connection.get_settings(
-        network.NM_CONNECTION_TYPE_802_11_OLPC_MESH)
-    if mesh_settings:
-        return not connection.get_id() in mesh_whitelist
-
-
 def clear_wireless_networks():
-    """Remove all wireless connections except Sugar-internal ones.
-    """
-    try:
-        connections = network.get_connections()
-    except dbus.DBusException:
-        logging.debug('NetworkManager not available')
-    else:
-        wireless_connections = \
-            (connection for connection in
-             connections.get_list() if is_wireless(connection))
-
-        for connection in wireless_connections:
-            try:
-                connection.delete()
-            except dbus.DBusException:
-                logging.debug("Could not remove connection %s",
-                              connection.get_id())
+    network.clear_wireless_networks()
 
 
 def have_wireless_networks():
-    """Check that there are non-Sugar-internal wireless connections.
-    """
-    try:
-        connections = network.get_connections()
-    except dbus.DBusException:
-        logging.debug('NetworkManager not available')
-        return False
-    else:
-        return any(is_wireless(connection)
-                   for connection in connections.get_list())
+    return network.have_wireless_networks()
 
 
 def get_publish_information():
@@ -194,8 +141,10 @@ def set_publish_information(value):
     try:
         value = (False, True)[int(value)]
     except:
-        raise ValueError(_('Error in specified argument use 0/1.'))
+        raise ValueError(_('Error in specified argument. Use 0/1.'))
 
     settings = Gio.Settings('org.sugarlabs.collaboration')
     settings.set_boolean('publish-gadget', value)
     return 0
+
+nm_client = NMClient.Client()
