@@ -61,7 +61,7 @@ class ViewLayout(Layout):
         self._height = allocation.height
         self._grid = Grid(int(allocation.width / _CELL_SIZE),
                           int(allocation.height / _CELL_SIZE))
-        self._grid.connect('child-changed', self.__grid_child_changed_cb)
+        self._grid.connect('child-changed', self.__grid_child_changed_cb, allocation)
         self._allocate_owner_icon(allocation, owner_icon, activity_icon)
 
     def _allocate_owner_icon(self, allocation, owner_icon, activity_icon):
@@ -145,12 +145,12 @@ class ViewLayout(Layout):
         height = math.ceil(request.height / _CELL_SIZE)
         return int(width), int(height)
 
-    def __grid_child_changed_cb(self, grid, child):
+    def __grid_child_changed_cb(self, grid, child, allocation):
         request = child.size_request()
         rect = self._grid.get_child_rect(child)
         child_allocation = Gdk.Rectangle()
         child_allocation.x = int(round(rect.x * _CELL_SIZE))
-        child_allocation.y = int(round(rect.y * _CELL_SIZE))
+        child_allocation.y = int(round(rect.y * _CELL_SIZE)) + allocation.y
         child_allocation.width = request.width
         child_allocation.height = request.height
         child.size_allocate(child_allocation)
@@ -285,8 +285,6 @@ class RandomLayout(SpreadLayout):
 
 
 _MINIMUM_RADIUS = style.XLARGE_ICON_SIZE / 2 + style.DEFAULT_SPACING
-_MAXIMUM_RADIUS = (Gdk.Screen.height() - style.GRID_CELL_SIZE) / 2 - \
-    style.DEFAULT_SPACING
 _RING_SPACING_FACTOR = 0.95
 _SPIRAL_SPACING_FACTOR = 0.75
 _RADIUS_GROWTH_FACTOR = 1.25
@@ -369,7 +367,9 @@ class RingLayout(ViewLayout):
 
     def _calculate_maximum_radius(self, icon_size):
         """ Return the maximum radius including encroachment. """
-        return _MAXIMUM_RADIUS - (icon_size * _MAXIMUM_RADIUS_PADDING_FACTOR)
+        r = (Gdk.Screen.height() - style.GRID_CELL_SIZE) / 2 - \
+                         style.DEFAULT_SPACING
+        return r - (icon_size * _MAXIMUM_RADIUS_PADDING_FACTOR)
 
     def _calculate_angle_and_radius(self, icon_count, icon_size):
         """ Based on icon_count and icon_size, calculate radius and angle. """
