@@ -1,8 +1,8 @@
 # Copyright (C) 2009, Tomeu Vizoso
 #
-# This program is free software; you can redistribute it and/or modify
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -11,8 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
 from gettext import gettext as _
@@ -23,6 +22,7 @@ from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import Pango
+from gi.repository import GdkPixbuf
 
 from sugar3.graphics import style
 from sugar3.graphics.icon import Icon, CellRendererIcon
@@ -39,6 +39,7 @@ from jarabe.journal import journalwindow
 
 
 UPDATE_INTERVAL = 300
+PROJECT_BUNDLE_ID = 'org.sugarlabs.Project'
 
 
 class TreeView(Gtk.TreeView):
@@ -49,6 +50,8 @@ class TreeView(Gtk.TreeView):
                            ([object])),
         'volume-error': (GObject.SignalFlags.RUN_FIRST, None,
                          ([str, str])),
+        'choose-project': (GObject.SignalFlags.RUN_FIRST, None,
+                          ([object])),
     }
 
     def __init__(self, journalactivity):
@@ -94,13 +97,26 @@ class TreeView(Gtk.TreeView):
                                     detail=True)
             palette.connect('detail-clicked', self.__detail_clicked_cb)
             palette.connect('volume-error', self.__volume_error_cb)
+            palette.connect('choose-project', self.__choose_project_cb)
 
         elif column in self.buddies_columns:
-            buddycellrenderer = column.get_cells()[0]
-            if buddycellrenderer.nick is not None:
-                palette = BuddyPalette(
-                    (buddycellrenderer.nick,
-                     buddycellrenderer.props.xo_color.to_string()))
+            tree_model = self.get_model()
+            iterator = tree_model.get_iter(path)
+
+            for column_index in [ListModel.COLUMN_BUDDY_1,
+                                 ListModel.COLUMN_BUDDY_2,
+                                 ListModel.COLUMN_BUDDY_3]:
+                if column == self.buddies_columns[column_index -
+                                                  ListModel.COLUMN_BUDDY_1]:
+                    buddy_value = tree_model.do_get_value(
+                        iterator,
+                        column_index
+                    )
+                    if buddy_value is not None:
+                        nick, xo_color = buddy_value
+                        palette = BuddyPalette(
+                            (nick,
+                             xo_color.to_string()))
 
         return palette
 
@@ -110,6 +126,9 @@ class TreeView(Gtk.TreeView):
     def __volume_error_cb(self, palette, message, severity):
         self.emit('volume-error', message, severity)
 
+    def __choose_project_cb(self, palette, metadata_to_copy):
+        self.emit('choose-project', metadata_to_copy)
+
     def do_size_request(self, requisition):
         # HACK: We tell the model that the view is just resizing so it can
         # avoid hitting both D-Bus and disk.
@@ -146,6 +165,7 @@ class BaseListView(Gtk.Bin):
         self._progress_bar = None
         self._last_progress_bar_pulse = None
         self._scroll_position = 0.
+        self._projects_view_active = False
 
         Gtk.Bin.__init__(self)
 
@@ -172,10 +192,10 @@ class BaseListView(Gtk.Bin):
         self.cell_icon = None
         self._title_column = None
         self.sort_column = None
-        self._add_columns()
-        scrolling_detector = ScrollingDetector(self._scrolled_window)
-        self.tree_view.connect_to_scroller(scrolling_detector)
+        self._scrolling_detector = ScrollingDetector(self._scrolled_window)
+        self.tree_view.connect_to_scroller(self._scrolling_detector)
 
+        self._add_columns()
         self.enable_drag_and_copy()
 
         # Auto-update stuff
@@ -191,10 +211,12 @@ class BaseListView(Gtk.Bin):
         model.deleted.connect(self.__model_deleted_cb)
 
     def enable_drag_and_copy(self):
-        self.tree_view.enable_model_drag_source(Gdk.ModifierType.BUTTON1_MASK,
-                                                [('text/uri-list', 0, 0),
-                                                 ('journal-object-id', 0, 0)],
-                                                Gdk.DragAction.COPY)
+        self.tree_view.drag_source_set(Gdk.ModifierType.BUTTON1_MASK,
+                                       [Gtk.TargetEntry.new(
+                                           'text/uri-list', 0, 0),
+                                        Gtk.TargetEntry.new(
+                                            'journal-object-id', 0, 0)],
+                                       Gdk.DragAction.COPY)
 
     def disable_drag_and_copy(self):
         self.tree_view.unset_rows_drag_source()
@@ -213,6 +235,13 @@ class BaseListView(Gtk.Bin):
 
     def _is_new_item_visible(self, object_id):
         """Check if the created item is part of the currently selected view"""
+        if 'project_id' in self._query:
+            # TODO:  Would be best to check if the object_id is in the project.
+            #        But there is only ever 1 project listview, so it should
+            #        not be very costly.
+            return True
+        if not self._query.get('mountpoints', None):
+            return None
         if self._query['mountpoints'] == ['/']:
             return not object_id.startswith('/')
         else:
@@ -235,15 +264,18 @@ class BaseListView(Gtk.Bin):
 
         cell_favorite = CellRendererFavorite()
         cell_favorite.connect('clicked', self._favorite_clicked_cb)
+        cell_favorite.connect_to_scroller(self._scrolling_detector)
 
-        column = Gtk.TreeViewColumn()
-        column.props.sizing = Gtk.TreeViewColumnSizing.FIXED
-        column.props.fixed_width = cell_favorite.props.width
-        column.pack_start(cell_favorite, True)
-        column.set_cell_data_func(cell_favorite, self.__favorite_set_data_cb)
-        self.tree_view.append_column(column)
+        self._fav_column = Gtk.TreeViewColumn()
+        self._fav_column.props.sizing = Gtk.TreeViewColumnSizing.FIXED
+        self._fav_column.props.fixed_width = cell_favorite.props.width
+        self._fav_column.pack_start(cell_favorite, True)
+        self._fav_column.set_cell_data_func(
+            cell_favorite, self.__favorite_set_data_cb)
+        self.tree_view.append_column(self._fav_column)
 
         self.cell_icon = CellRendererActivityIcon()
+        self.cell_icon.connect_to_scroller(self._scrolling_detector)
 
         column = Gtk.TreeViewColumn()
         self.tree_view.icon_activity_column = column
@@ -343,7 +375,6 @@ class BaseListView(Gtk.Bin):
         if buddy is None:
             cell.props.visible = False
             return
-        logging.error('__buddies_set_data_cb %s', buddy)
         # FIXME workaround for pygobject bug, see
         # https://bugzilla.gnome.org/show_bug.cgi?id=689277
         #
@@ -369,14 +400,20 @@ class BaseListView(Gtk.Bin):
 
     def _favorite_clicked_cb(self, cell, path):
         row = self._model[path]
+        iterator = self._model.get_iter(path)
         metadata = model.get(row[ListModel.COLUMN_UID])
         if not model.is_editable(metadata):
             return
         if metadata.get('keep', 0) == '1':
             metadata['keep'] = '0'
+            self._model[iterator][ListModel.COLUMN_FAVORITE] = '0'
         else:
             metadata['keep'] = '1'
-        model.write(metadata, update_mtime=False)
+            self._model[iterator][ListModel.COLUMN_FAVORITE] = '1'
+
+        cell_rect = self.tree_view.get_cell_area(path, self._fav_column)
+        self.tree_view.queue_draw_area(cell_rect.x, cell_rect.y,
+                                       cell_rect.width, cell_rect.height)
 
     def __select_set_data_cb(self, column, cell, tree_model, tree_iter,
                              data):
@@ -393,6 +430,7 @@ class BaseListView(Gtk.Bin):
 
     def update_with_query(self, query_dict):
         logging.debug('ListView.update_with_query')
+
         if 'order_by' not in query_dict:
             query_dict['order_by'] = ['+timestamp']
         if query_dict['order_by'] != self._query.get('order_by'):
@@ -426,7 +464,7 @@ class BaseListView(Gtk.Bin):
         self._model = ListModel(self._query)
         self._model.connect('ready', self.__model_ready_cb)
         self._model.connect('progress', self.__model_progress_cb)
-        self._model.setup()
+        self._model.setup(self.__model_updated_cb)
         window = self.get_toplevel().get_window()
         if window is not None:
             window.set_cursor(None)
@@ -460,18 +498,25 @@ class BaseListView(Gtk.Bin):
             self.tree_view.get_bin_window().show()
 
         if len(tree_model) == 0:
-            documents_path = model.get_documents_path()
-            if self._is_query_empty():
-                if self._query['mountpoints'] == ['/']:
-                    self._show_message(_('Your Journal is empty'))
-                elif documents_path and self._query['mountpoints'] == \
-                        [documents_path]:
-                    self._show_message(_('Your documents folder is empty'))
-                else:
-                    self._show_message(_('The device is empty'))
+            if self._query.get('project_id', None):
+                self._show_message(_('Your project is empty'))
             else:
-                self._show_message(_('No matching entries'),
-                                   show_clear_query=self._can_clear_query())
+                documents_path = model.get_documents_path()
+                if self._is_query_empty():
+                    if self._query['mountpoints'] == ['/']:
+                        self._show_message(_('Your Journal is empty'))
+                    elif documents_path and self._query['mountpoints'] == \
+                            [documents_path]:
+                        self._show_message(_('Your documents folder is empty'))
+                    else:
+                        self._show_message(_('The device is empty'))
+                else:
+                    show_message_text = _('No matching entries')
+                    if self.get_projects_view_active():
+                        show_message_text = _('No Projects')
+
+                    self._show_message(show_message_text,
+                        show_clear_query=self._can_clear_query())
         else:
             self._clear_message()
 
@@ -550,17 +595,19 @@ class BaseListView(Gtk.Bin):
             color, GLib.markup_escape_text(message)))
         box.pack_start(label, expand=True, fill=False, padding=0)
 
-        if show_clear_query:
-            button_box = Gtk.HButtonBox()
-            button_box.set_layout(Gtk.ButtonBoxStyle.CENTER)
-            box.pack_start(button_box, False, True, 0)
-            button_box.show()
+        if not self.get_projects_view_active():
+            if show_clear_query:
+                button_box = Gtk.HButtonBox()
+                button_box.set_layout(Gtk.ButtonBoxStyle.CENTER)
+                box.pack_start(button_box, False, True, 0)
+                button_box.show()
 
-            button = Gtk.Button(label=_('Clear search'))
-            button.connect('clicked', self.__clear_button_clicked_cb)
-            button.props.image = Icon(icon_name='dialog-cancel',
-                                      pixel_size=style.SMALL_ICON_SIZE)
-            button_box.pack_start(button, expand=True, fill=False, padding=0)
+                button = Gtk.Button(label=_('Clear search'))
+                button.connect('clicked', self.__clear_button_clicked_cb)
+                button.props.image = Icon(icon_name='dialog-cancel',
+                                          pixel_size=style.SMALL_ICON_SIZE)
+                button_box.pack_start(button, expand=True, fill=False,
+                                      padding=0)
 
         background_box.show_all()
 
@@ -657,6 +704,9 @@ class BaseListView(Gtk.Bin):
     def __volume_error_cb(self, palette, message, severity):
         self.emit('volume-error', message, severity)
 
+    def get_projects_view_active(self):
+        return self._query.get('activity') == PROJECT_BUNDLE_ID
+
 
 class ListView(BaseListView):
     __gtype_name__ = 'JournalListView'
@@ -666,6 +716,8 @@ class ListView(BaseListView):
                                ([])),
         'title-edit-finished': (GObject.SignalFlags.RUN_FIRST, None,
                                 ([])),
+        'project-view-activate': (GObject.SignalFlags.RUN_FIRST, None,
+                                 ([object])),
     }
 
     def __init__(self, journalactivity, enable_multi_operations=False):
@@ -695,6 +747,13 @@ class ListView(BaseListView):
         return self._is_dragging
 
     def __drag_begin_cb(self, widget, drag_context):
+        path, _column = self.tree_view.get_cursor()
+        if path is None:
+            return
+
+        row = self.tree_view.get_model()[path]
+        _pixbuf = GdkPixbuf.Pixbuf.new_from_file(row[ListModel.COLUMN_ICON])
+        self.tree_view.drag_source_set_icon_pixbuf(_pixbuf)
         self._is_dragging = True
 
     def __drag_data_get_cb(self, widget, context, selection, info, time):
@@ -735,15 +794,15 @@ class ListView(BaseListView):
     def __icon_clicked_cb(self, cell, path):
         row = self.tree_view.get_model()[path]
         metadata = model.get(row[ListModel.COLUMN_UID])
+        if metadata['activity'] == PROJECT_BUNDLE_ID:
+            self.emit('project-view-activate', metadata)
+            return
         misc.resume(metadata,
                     alert_window=journalwindow.get_journal_window())
 
     def __cell_title_edited_cb(self, cell, path, new_text):
-        row = self._model[path]
-        metadata = model.get(row[ListModel.COLUMN_UID])
-        metadata['title'] = new_text
-        model.write(metadata, update_mtime=False)
-        self.cell_title.props.editable = False
+        iterator = self._model.get_iter(path)
+        self._model[iterator][ListModel.COLUMN_TITLE] = new_text
         self.emit('title-edit-finished')
 
     def __editing_canceled_cb(self, cell):
