@@ -1,8 +1,8 @@
 # Copyright (C) 2008 Tomeu Vizoso
 #
-# This program is free software; you can redistribute it and/or modify
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -11,8 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
 import logging
@@ -22,7 +21,10 @@ from gi.repository import GObject
 from gi.repository import Gio
 from gi.repository import GLib
 import dbus
-from telepathy.interfaces import CONNECTION_INTERFACE_REQUESTS, CHANNEL
+from telepathy.interfaces import \
+    CONNECTION_INTERFACE_REQUESTS, \
+    CHANNEL, \
+    CHANNEL_DISPATCHER
 from telepathy.constants import CONNECTION_HANDLE_TYPE_CONTACT,     \
     SOCKET_ADDRESS_TYPE_UNIX,           \
     SOCKET_ACCESS_CONTROL_LOCALHOST
@@ -32,6 +34,7 @@ from sugar3.presence import presenceservice
 from sugar3 import dispatch
 
 from jarabe.util.telepathy import connection_watcher
+from jarabe.model import telepathyclient
 from jarabe.model import neighborhood
 
 
@@ -132,6 +135,7 @@ class BaseFileTransfer(GObject.GObject):
 
 
 class IncomingFileTransfer(BaseFileTransfer):
+
     def __init__(self, connection, object_path, props):
         BaseFileTransfer.__init__(self, connection)
 
@@ -184,6 +188,7 @@ class IncomingFileTransfer(BaseFileTransfer):
 
 
 class OutgoingFileTransfer(BaseFileTransfer):
+
     def __init__(self, buddy, file_name, title, description, mime_type):
 
         presence_service = presenceservice.get_instance()
@@ -277,6 +282,20 @@ def _connection_removed_cb(conn_watcher, connection):
     logging.debug('connection removed %r', connection)
 
 
+def _got_dispatch_operation_cb(**kwargs):
+    dispatch_operation_path = kwargs['dispatch_operation_path']
+    channel_path, channel_properties = kwargs['channels'][0]
+    channel_type = channel_properties[CHANNEL + '.ChannelType']
+    handle_type = channel_properties[CHANNEL + '.TargetHandleType']
+
+    if handle_type == CONNECTION_HANDLE_TYPE_CONTACT and \
+       channel_type == CHANNEL_TYPE_FILE_TRANSFER:
+        # We must claim our file transfers so that empathy doesn't get it
+        bus = dbus.Bus()
+        operation = bus.get_object(CHANNEL_DISPATCHER, dispatch_operation_path)
+        operation.Claim()
+
+
 def init():
     conn_watcher = connection_watcher.get_instance()
     conn_watcher.connect('connection-added', _connection_added_cb)
@@ -285,6 +304,9 @@ def init():
     for connection in conn_watcher.get_connections():
         _monitor_connection(connection)
 
+    client_handler = telepathyclient.get_instance()
+    client_handler.got_dispatch_operation.connect(_got_dispatch_operation_cb)
+
 
 def start_transfer(buddy, file_name, title, description, mime_type):
     outgoing_file_transfer = OutgoingFileTransfer(buddy, file_name, title,
