@@ -1,9 +1,10 @@
 # Copyright (C) 2013 Kalpa Welivitigoda
-# Copyright (C) 2015 Sam Parkinson
+# Copyright (C) 2015-2016 Sam Parkinson
+# Copyright (C) 2016 James Cameron <quozl@laptop.org>
 #
-# This program is free software; you can redistribute it and/or modify
+# This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
@@ -12,8 +13,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from gettext import gettext as _
 import logging
@@ -23,21 +23,29 @@ import json
 from gi.repository import Gtk
 from gi.repository import GObject
 from gi.repository import Gdk
-from gi.repository import WebKit
 from gi.repository import GdkX11
-from gi.repository import SoupGNOME
 from gi.repository import Gio
 
 from sugar3 import env
 from sugar3.graphics import style
 from sugar3.graphics.icon import Icon
 from sugar3.graphics.toolbutton import ToolButton
-from sugar3.graphics.icon import get_icon_file_name
 from sugar3.graphics.radiotoolbutton import RadioToolButton
 from sugar3.bundle.activitybundle import get_bundle_instance
 from jarabe.model import shell
 
 
+try:
+    olpc_build = file('/boot/olpc_build', 'r').readline()
+except:
+    olpc_build = ''
+
+if olpc_build.startswith('13'):
+    from jarabe.view.viewhelp_webkit1 import Browser
+else:
+    from jarabe.view.viewhelp_webkit2 import Browser
+
+
 _logger = logging.getLogger('ViewHelp')
 
 _MODE_HELP = 0
@@ -46,6 +54,11 @@ _MODE_SOCIAL_HELP = 1
 _LOADING_ICON = 'toolbar-social-help-animated'
 
 
+def _get_current_language():
+    locale = os.environ.get('LANG')
+    return locale.split('.')[0].split('_')[0].lower()
+
+
 def _get_help_activity_path():
     path = os.path.join(env.get_user_activities_path(), 'Help.activity')
     if os.path.exists(path):
@@ -156,6 +169,7 @@ class ViewHelp(Gtk.Window):
         self.add(box)
         box.show()
 
+        self.set_modal(True)
         self.set_decorated(False)
         self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
         self.set_border_width(style.LINE_WIDTH)
@@ -169,53 +183,32 @@ class ViewHelp(Gtk.Window):
         self.connect('hide', self.__hide_cb)
         self.connect('key-press-event', self.__key_press_event_cb)
 
-        toolbar = Toolbar(title, has_local_help)
-        box.pack_start(toolbar, False, False, 0)
-        toolbar.show()
-        toolbar.connect('stop-clicked', self.__stop_clicked_cb)
-        toolbar.connect('mode-changed', self.__mode_changed_cb)
+        self._toolbar = Toolbar(title, has_local_help)
+        box.pack_start(self._toolbar, False, False, 0)
+        self._toolbar.show()
+        self._toolbar.connect('stop-clicked', self.__stop_clicked_cb)
+        self._toolbar.connect('mode-changed', self.__mode_changed_cb)
 
-        session = WebKit.get_default_session()
-        cookie_jar = SoupGNOME.CookieJarSqlite(
-            filename=os.path.join(env.get_profile_path(),
-                                  'social-help.cookies'),
-            read_only=False)
-        session.add_feature(cookie_jar)
-
-        self._webview = WebKit.WebView()
-        self._webview.set_full_content_zoom(True)
-        self._webview.connect('resource-request-starting',
-                              self._resource_request_starting_cb)
+        self._browser = Browser(self._toolbar)
 
         scrolled_window = Gtk.ScrolledWindow()
-        scrolled_window.add(self._webview)
+        scrolled_window.add(self._browser.get_widget())
         scrolled_window.show()
 
         box.pack_start(scrolled_window, True, True, 0)
-        self._webview.show()
 
-        language = self._get_current_language()
+        language = _get_current_language()
+        self._help_state = None
         if has_local_help:
-            self._help_url = 'file://' + self._get_help_file(language, url)
-        self._social_help_url = '{}/goto/{}'.format(
+            self._help_url = self._browser.get_local_method() + \
+                             self._get_help_file(language, url)
+
+        self._social_help_uri = '{}/goto/{}'.format(
             get_social_help_server(), activity.get_bundle_id())
+        self._social_help_state = None
 
-        self._webview.connect(
-            'notify::load-status', self.__load_status_changed_cb)
         self._load_mode(self._mode)
 
-    def _resource_request_starting_cb(self, webview, web_frame, web_resource,
-                                      request, response):
-        uri = web_resource.get_uri()
-        if uri.startswith('file://') and uri.find('_images') > -1:
-            if uri.find('/%s/_images/' % self._get_current_language()) > -1:
-                new_uri = uri.replace('/html/%s/_images/' %
-                                      self._get_current_language(),
-                                      '/images/')
-            else:
-                new_uri = uri.replace('/html/_images/', '/images/')
-            request.set_uri(new_uri)
-
     def __stop_clicked_cb(self, widget):
         self.destroy()
 
@@ -224,52 +217,35 @@ class ViewHelp(Gtk.Window):
             self.__stop_clicked_cb(None)
 
     def __mode_changed_cb(self, toolbar, mode):
-        if self._mode == _MODE_HELP:
-            self._help_url = self._webview.props.uri
+        if mode == _MODE_HELP:
+            self._social_help_state = self._browser.save_state()
         else:
-            self._social_help_url = self._webview.props.uri
+            self._help_state = self._browser.save_state()
 
         self._mode = mode
         self._load_mode(self._mode)
 
     def _load_mode(self, mode):
         if mode == _MODE_HELP:
-            self._webview.load_uri(self._help_url)
+            self._browser.load_state(self._help_state, self._help_url)
         else:
-            # Loading any content for the social help page can take a
-            # very long time (eg. the site is behind a redirector).
-            # Loading the animation forces webkit to re-render the
-            # page instead of keeping the previous page (so the user
-            # sees that it is loading)
-            path = get_icon_file_name(_LOADING_ICON)
-            if path:
-                self._webview.load_uri('file://' + path)
-                # Social help is loaded after the icon is loaded
-            else:
-                self._webview.load_uri(self._social_help_url)
-
-    def __load_status_changed_cb(self, *args):
-        if self._webview.props.load_status == WebKit.LoadStatus.FINISHED \
-           and _LOADING_ICON in self._webview.props.uri:
-            self._webview.load_uri(self._social_help_url)
+            self._browser.load_state(self._social_help_state,
+                                     self._social_help_uri)
 
     def __realize_cb(self, widget):
         self.set_type_hint(Gdk.WindowTypeHint.DIALOG)
         window = self.get_window()
         window.set_accept_focus(True)
-        display = Gdk.Display.get_default()
-        parent = GdkX11.X11Window.foreign_new_for_display(
-            display, self.parent_window_xid)
-        window.set_transient_for(parent)
+        if self.parent_window_xid > 0:
+            display = Gdk.Display.get_default()
+            parent = GdkX11.X11Window.foreign_new_for_display(
+                display, self.parent_window_xid)
+            window.set_transient_for(parent)
         shell.get_model().push_modal()
 
     def __hide_cb(self, widget):
         shell.get_model().pop_modal()
 
-    def _get_current_language(self):
-        locale = os.environ.get('LANG')
-        return locale.split('.')[0].split('_')[0].lower()
-
     def _get_help_file(self, language, help_file):
         activity_path = _get_help_activity_path()
         # check if exist a page for the language selected
@@ -284,6 +260,8 @@ class ViewHelp(Gtk.Window):
 class Toolbar(Gtk.Toolbar):
 
     __gsignals__ = {
+        'back-clicked': (GObject.SignalFlags.RUN_FIRST, None, ([])),
+        'forward-clicked': (GObject.SignalFlags.RUN_FIRST, None, ([])),
         'stop-clicked': (GObject.SignalFlags.RUN_FIRST, None, ([])),
         'mode-changed': (GObject.SignalFlags.RUN_FIRST, None, ([int])),
     }
@@ -317,12 +295,23 @@ class Toolbar(Gtk.Toolbar):
             icon.show()
             social_help_button.props.tooltip = _('Social Help')
             social_help_button.props.group = help_button
-            social_help_button.connect('toggled', self.__button_toggled_cb,
-                                       _MODE_SOCIAL_HELP)
+            social_help_button.connect(
+                'toggled', self.__button_toggled_cb, _MODE_SOCIAL_HELP)
             self.insert(social_help_button, -1)
             social_help_button.show()
             self._add_separator(False)
 
+        self._back_button = ToolButton(icon_name='go-previous-paired')
+        self._back_button.props.tooltip = _('Back')
+        self._back_button.connect('clicked', self.__back_clicked_cb)
+        self.insert(self._back_button, -1)
+        self._back_button.show()
+        self._forward_button = ToolButton(icon_name='go-next-paired')
+        self._forward_button.props.tooltip = _('Forward')
+        self._forward_button.connect('clicked', self.__forward_clicked_cb)
+        self.insert(self._forward_button, -1)
+        self._forward_button.show()
+
         title = _('Help: %s') % activity_name
         self._label = Gtk.Label()
         self._label.set_markup('<b>%s</b>' % title)
@@ -360,3 +349,13 @@ class Toolbar(Gtk.Toolbar):
             separator.set_size_request(style.DEFAULT_SPACING, -1)
         self.insert(separator, -1)
         separator.show()
+
+    def update_back_forward(self, can_go_back, can_go_forward):
+        self._back_button.props.sensitive = can_go_back
+        self._forward_button.props.sensitive = can_go_forward
+
+    def __back_clicked_cb(self, button):
+        self.emit('back-clicked')
+
+    def __forward_clicked_cb(self, button):
+        self.emit('forward-clicked')
